import logging
import random
from functools import wraps

import requests

from rollbot import as_plugin, RollbotFailure

from config import get_secret

TS3_QUERY = get_secret("teamspeak.query")


def with_error_handle(f):
    @wraps(f)
    def wrapper():
        try:
            return f()
        except ConnectionError as _:
            return RollbotFailure.SERVICE_DOWN
    return wrapper


def fmt_list(nicks):
    if len(nicks) == 0:
        return "No one in teamspeak!"
    if len(nicks):
        return ("Only " if random.randint(0, 1) == 0 else "Oops all ") + nicks[0]
    return "I see the following people: " + ", ".join(nicks)


@as_plugin
@with_error_handle
def teamspeak():
    nicks = requests.get(TS3_QUERY).json()["users"]
    return fmt_list(nicks)


@as_plugin
@with_error_handle
def teemspeem():
    nicks = requests.get(TS3_QUERY).json()["users"]
    fmtd = fmt_list(nicks)
    for v in "aiou":
        fmtd = fmtd.replace(v, "e").replace(v.upper(), "E")
    return fmtd.replace("k", "m").replace("K", "M")


@as_plugin
@with_error_handle
def speamteek():
    nicks = requests.get(TS3_QUERY).json()["users"]
    return fmt_list(nicks)[::-1]


@as_plugin
@with_error_handle
def teamscream():
    nicks = requests.get(TS3_QUERY).json()["users"]
    return fmt_list(nicks).upper()


@as_plugin
@with_error_handle
def ts():
    nicks = requests.get(TS3_QUERY).json()["users"]
    initials = [nick[0] for nick in nicks]
    if len(initials) == 0:
        return ""
    return "i c: " + ", ".join(initials)
